"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const procurator_1 = require("procurator");
const stix_1 = require("stix");
const stix_wetland_1 = require("stix-wetland");
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
const stix_gates_1 = require("stix-gates");
const stix_security_1 = require("stix-security");
class SwaggerController extends stix_1.AbstractActionController {
    async ui() {
        const ui = fs
            .createReadStream(path.resolve(__dirname, '..', '..', '..', 'static', 'index.html'))
            .pipe(procurator_1.stream({ url: this.serverService.getURL() + 'swagger/doc' }, false));
        return this.okResponse({}).html(ui);
    }
    async doc() {
        const queryParameters = {};
        const paths = {};
        const tags = {};
        const components = {
            securitySchemes: {},
            schemas: {
                Error: {
                    required: ['code', 'message'],
                    properties: {
                        data: {
                            type: 'object',
                        },
                        message: {
                            type: 'string',
                        },
                    },
                },
            },
        };
        const routeToSwaggerPath = (route) => route.replace(/(:)(.*?)(\/|$)/g, '{$2}$3');
        const ensureSchema = (controller, method) => {
            const Entity = stix_wetland_1.getAssociatedEntity(controller);
            if (!Entity) {
                return;
            }
            const mapping = this.wetlandService.getMapping(Entity);
            const schemaKey = (method === stix_1.RequestMethods.Post ? 'New' : '') + mapping.getEntityName();
            if (!components.schemas[schemaKey]) {
                components.schemas[schemaKey] = makeEntitySchema(mapping);
            }
            return schemaKey;
        };
        const makeBody = (controller, schemaKey) => {
            return {
                description: 'Data model for ' + controller.name,
                required: true,
                content: {
                    'application/json': {
                        schema: {
                            $ref: '#/components/schemas/' + schemaKey,
                        },
                    },
                },
            };
        };
        const makeResponses = (resourceName, schemaKey, method) => {
            const resourceResonse = {
                description: resourceName + ' response',
            };
            if (schemaKey) {
                resourceResonse.content = {
                    'application/json': {
                        schema: {
                            $ref: '#/components/schemas/' + schemaKey,
                        },
                    },
                };
            }
            return {
                [method === stix_1.RequestMethods.Post ? '201' : '200']: resourceResonse,
                default: {
                    description: 'unexpected error',
                    content: {
                        'application/json': {
                            schema: {
                                $ref: '#/components/schemas/Error',
                            },
                        },
                    },
                },
            };
        };
        const allowedTypes = ['array', 'boolean', 'integer', 'number', 'object', 'string'];
        const makeEntitySchema = (mapping) => {
            const fields = mapping.getFields();
            const scheme = {
                required: [],
                properties: {},
            };
            const entityParameters = [];
            Object.keys(fields).forEach(property => {
                const field = fields[property];
                if (field.primary) {
                    return;
                }
                const required = !field.nullable && !field.defaultTo && !field.generatedValue;
                const swaggerType = allowedTypes.indexOf(field.type) === -1 ? 'string' : field.type;
                if (required) {
                    scheme.required.push(property);
                }
                scheme.properties[property] = {
                    type: swaggerType,
                };
                entityParameters.push({
                    name: property,
                    optional: true,
                    type: swaggerType,
                });
            });
            queryParameters[mapping.getEntityName()] = makeParameters(entityParameters, 'query');
            return scheme;
        };
        const requestMethodsWithBody = [stix_1.RequestMethods.Post, stix_1.RequestMethods.Put, stix_1.RequestMethods.Patch];
        const makePaths = (routes) => {
            routes.forEach((route) => {
                const routeKey = routeToSwaggerPath(route.route);
                const method = route.method;
                const schemaKey = ensureSchema(route.controller, method);
                const tagName = route.controller.name.replace(/Controller$/, '');
                paths[routeKey] = paths[routeKey] || {};
                tags[tagName] = {
                    name: tagName,
                    description: `Routes for ${tagName}`,
                };
                const path = (paths[routeKey][route.method.toLowerCase()] = {});
                const controller = this.controllerManager.get(route.controller);
                const applicableGates = [].concat(stix_gates_1.Gate.applicableGates(controller, route.action, this.gateConfig.rules));
                const security = applicableGates.reduce((securitySchemes, gate) => {
                    if (gate.prototype && gate.prototype instanceof stix_security_1.SecurityGate) {
                        const key = gate.getConfigKey();
                        components.securitySchemes[key] = this.securityConfig.schemes[key].scheme;
                        securitySchemes[key] = [];
                    }
                    return securitySchemes;
                }, {});
                if (Object.keys(security).length) {
                    path.security = [security];
                }
                Object.assign(path, {
                    tags: [tagName],
                    description: `${route.action}`,
                    operationId: [route.method, route.controller.name, route.action].join('_'),
                    responses: makeResponses(tagName, schemaKey, method),
                });
                const parameters = [];
                if (route.action === 'find') {
                    try {
                        const Entity = stix_wetland_1.getAssociatedEntity(route.controller);
                        if (!Entity) {
                            return;
                        }
                        const mapping = this.wetlandService.getMapping(Entity);
                        parameters.push(...queryParameters[mapping.getEntityName()]);
                    }
                    catch (error) {
                        // Lol, ignored!
                    }
                }
                if (Array.isArray(route.keys) && route.keys.length) {
                    parameters.push(...makeParameters(route.keys));
                }
                if (parameters.length) {
                    path.parameters = parameters;
                }
                if (requestMethodsWithBody.indexOf(method) > -1) {
                    path.requestBody = makeBody(route.controller, schemaKey);
                }
            });
            return paths;
        };
        const makeParameters = (keys, availableIn = 'path') => {
            return keys.map((key) => ({
                name: key.name,
                in: availableIn,
                description: key.name + ' value.',
                required: !key.optional,
                schema: {
                    type: key.type || 'string',
                },
            }));
        };
        const gitSumSwag = () => {
            return {
                openapi: '3.0.0',
                info: {
                    version: '1.0.0',
                    title: 'Stix API generated swagger docs.',
                    description: 'Docs generated with some late-night written hacky garbage code.',
                    contact: {
                        name: 'RWOverdijk',
                        email: 'wesley@spoonx.studio',
                        url: 'https://spoonx.studio/',
                    },
                },
                tags: Object.values(tags),
                servers: [
                    {
                        url: this.serverService.getURL(),
                    },
                ],
                components,
                paths: makePaths(this.routerService.getRegisteredRoutes()),
            };
        };
        return this.okResponse(gitSumSwag());
    }
}
__decorate([
    stix_1.inject(stix_1.RouterService)
], SwaggerController.prototype, "routerService", void 0);
__decorate([
    stix_1.inject(stix_wetland_1.WetlandService)
], SwaggerController.prototype, "wetlandService", void 0);
__decorate([
    stix_1.inject(stix_1.Config)
], SwaggerController.prototype, "config", void 0);
__decorate([
    stix_1.config('gate')
], SwaggerController.prototype, "gateConfig", void 0);
__decorate([
    stix_1.config('security')
], SwaggerController.prototype, "securityConfig", void 0);
__decorate([
    stix_1.inject(stix_1.ControllerManager)
], SwaggerController.prototype, "controllerManager", void 0);
__decorate([
    stix_1.inject(stix_1.ServerService)
], SwaggerController.prototype, "serverService", void 0);
exports.SwaggerController = SwaggerController;
//# sourceMappingURL=SwaggerController.js.map